package com.yourkit.probes.builtin;

import com.yourkit.probes.*;
import com.yourkit.runtime.Callback;
import org.jetbrains.annotations.Nullable;

import java.awt.event.InputEvent;
import java.awt.event.InvocationEvent;

/**
 * Detect long running AWT/Swing events
 */
public final class AWTEvents {
  public static final String TABLE_NAME = "Long AWT Event";

  private static final class EventTable extends Table {
    private final ClassNameColumn myEventClass = new ClassNameColumn("Event Class");
    private final ClassNameColumn myRunnableClass = new ClassNameColumn("Runnable Class");

    public EventTable() {
      super(
        AWTEvents.class,
        TABLE_NAME,

        // It makes no sense to record stack trace

        Table.LASTING_EVENTS |
        Table.RECORD_THREAD
      );
    }
  }
  private static final EventTable T_EVENT = new EventTable();
  static {
    // Record only events longer than 300 milliseconds
    T_EVENT.setMinimumRecordedLastingEventTime(300);
  }

  @MethodPattern("java.awt.EventQueue:dispatchEvent(java.awt.AWTEvent)")
  public static final class EventQueue_dispatchEvent_Probe {
    public static int onEnter(@Param(1) final Object event) {
      if (event instanceof InputEvent) {
        // these events are of no interest
        return Table.NO_ROW;
      }

      return T_EVENT.createRow();
    }

    public static void onExit(
      @OnEnterResult final int rowIndex,
      @Param(1) final Object event,
      @ThrownException @Nullable final Throwable e
    ) {
      T_EVENT.myEventClass.setValue(rowIndex, event.getClass());

      if (event instanceof InvocationEvent) {
        try {
          final Object runnableValue = Callback.getFieldObjectValue(event, "runnable", Runnable.class);
          if (runnableValue != null) {
            T_EVENT.myRunnableClass.setValue(rowIndex, runnableValue.getClass());
          }
        }
        catch (final Throwable ignored) {
        }
      }

      T_EVENT.closeRow(rowIndex, e);
    }
  }
}
